# Version scheme:
# YEAR.MONTH.COUNTER
# YEAR, MONTH are two digits and zero-padded
# COUNTER resets every month and starts with 0
# For example:
# - 23.09.0
# - 23.09.1
# - 23.10.0
#
# Development versions contain the DEV-COUNTER postfix:
# - 23.09.0.DEV-1

__version__ = '23.09.1'
